/**
 * ========================================
 * <libname> : a free Java <foobar> library
 * ========================================
 *
 * Project Info:  http://www.jfree.org/liblayout/
 * Project Lead:  Thomas Morgner;
 *
 * (C) Copyright 2005, by Object Refinery Limited and Contributors.
 *
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation;
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * ---------
 * ParameterDataRow.java
 * ---------
 *
 * Original Author:  Thomas Morgner;
 * Contributors: -;
 *
 * $Id: ParameterDataRow.java,v 1.1.2.2 2006/07/08 13:57:02 taqua Exp $
 *
 * Changes
 * -------------------------
 * 27.06.2006 : Initial version
 */
package org.jfree.report;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.jfree.report.util.ReportProperties;

/**
 * Creation-Date: 27.06.2006, 11:55:11
 *
 * @author Thomas Morgner
 */
public class ParameterDataRow implements DataRow
{
  private HashMap parameters;
  private ArrayList columns;


  public ParameterDataRow(final ReportProperties base)
  {
    if (base == null)
    {
      throw new NullPointerException();
    }
    this.columns = new ArrayList();
    this.parameters = new HashMap();

    final Iterator keys = base.keys();
    while (keys.hasNext())
    {
      final String key = (String) keys.next();
      if (base.isMarked(key))
      {
        columns.add(key);
        parameters.put(key, base.get(key));
      }
    }
  }

  /**
   * Returns the value of the expression or column in the tablemodel using the
   * given column number as index. For functions and expressions, the
   * <code>getValue()</code> method is called and for columns from the
   * tablemodel the tablemodel method <code>getValueAt(row, column)</code> gets
   * called.
   *
   * @param col the item index.
   * @return the value.
   * @throws IllegalStateException if the datarow detected a deadlock.
   */
  public Object get(int col)
  {
    if (col < columns.size() && col >= 0)
    {
      return parameters.get(columns.get(col));
    }
    return null;
  }

  /**
   * Returns the value of the function, expression or column using its specific
   * name. The given name is translated into a valid column number and the the
   * column is queried. For functions and expressions, the
   * <code>getValue()</code> method is called and for columns from the
   * tablemodel the tablemodel method <code>getValueAt(row, column)</code> gets
   * called.
   *
   * @param col the item index.
   * @return the value.
   * @throws IllegalStateException if the datarow detected a deadlock.
   */
  public Object get(String col) throws IllegalStateException
  {
    return parameters.get(col);
  }

  /**
   * Returns the name of the column, expression or function. For columns from
   * the tablemodel, the tablemodels <code>getColumnName</code> method is
   * called. For functions, expressions and report properties the assigned name
   * is returned.
   *
   * @param col the item index.
   * @return the name.
   */
  public String getColumnName(int col)
  {
    if (col < columns.size() && col >= 0)
    {
      return (String) columns.get(col);
    }
    return null;
  }

  /**
   * Returns the column position of the column, expression or function with the
   * given name or -1 if the given name does not exist in this DataRow.
   *
   * @param name the item name.
   * @return the item index.
   */
  public int findColumn(String name)
  {
    return columns.indexOf(name);
  }

  /**
   * Returns the number of columns, expressions and functions and marked
   * ReportProperties in the report.
   *
   * @return the item count.
   */
  public int getColumnCount()
  {
    return columns.size();
  }
}

