/**
 * ========================================
 * JFreeReport : a free Java report library
 * ========================================
 *
 * Project Info:  http://www.jfree.org/jfreereport/index.html
 * Project Lead:  Thomas Morgner;
 *
 * (C) Copyright 2000-2003, by Simba Management Limited and Contributors.
 *
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation;
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * -------------------------
 * ElementTrafficLightFunction
 * -------------------------
 * (C)opyright 2003, by Thomas Morgner and Contributors.
 *
 * Original Author:  Thomas Morgner;
 * Contributor(s):   David Gilbert (for Simba Management Limited);
 *
 */
package org.jfree.report.function;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Arrays;

import org.jfree.report.Band;
import org.jfree.report.Element;
import org.jfree.report.style.ElementStyleSheet;

/**
 * A function that performs basic traffic lighting based on a range of values
 * and a given set of colors to use.  The default colors are for
 * red/yellow/green and may be overridden.  This function respects absolute
 * values when flagged.
 * <p>
 * By default the given limits specify the lower boundary of an range. That
 * means a value lower than the first given limit will return the default color,
 * a value lower than the second value will return the first color and so on.
 * <p>
 * That logic can be inversed using the 'useOppositeLogic' flag. In that case
 * the limits specify an upper boundary. If the value read from the datarow
 * is greater than the last limit specified, the default is returned. If the
 * value is greater than the second last limit, the last color is used, and
 * so on.
 *
 * @author Michael D'Amour
 */
public class ElementTrafficLightFunction extends AbstractElementFormatFunction
{
  /** the color if the field is TRUE. */
  private boolean useOppositeLogic;
  private boolean useAbsoluteValue;
  private String field;
  private Color defaultColor;

  private ArrayList colors;
  private ArrayList values;

  /** Default constructor. */
  public ElementTrafficLightFunction()
  {
    colors = new ArrayList();
    values = new ArrayList();
  }

  private void configureDefaultBehaviour ()
  {
    if (colors.size() > 0 || values.size() > 0)
    {
      return;
    }

    if (defaultColor == null)
    {
      defaultColor = Color.red;
    }
    if (colors.size() == 0)
    {
      colors.add(Color.yellow);
      colors.add(Color.green);
    }
    if (values.size() == 0)
    {
      values.add(new Double(50));
      values.add(new Double(75));
    }
  }

  /**
   * Returns the field used by the function.
   * <p/>
   * The field name corresponds to a column name in the report's TableModel.
   *
   * @return The field name.
   */
  public String getField()
  {
    return field;
  }

  /**
   * Sets the field name for the function.
   * <p/>
   * The field name corresponds to a column name in the report's TableModel.
   *
   * @param field the field name (null not permitted).
   */
  public void setField(final String field)
  {
    if (field == null)
    {
      throw new NullPointerException();
    }
    this.field = field;
  }

  /**
   * Process the given band and color the named element of that band if it
   * exists.
   *
   * @param b the band that should be colored.
   */
  protected void processRootBand(final Band b)
  {
    // only if needed ...
    configureDefaultBehaviour();

    final Element[] elements = FunctionUtilities.findAllElements(b,
            getElement());
    if (elements.length == 0)
    {
      // there is no such element ! (we searched it by its name)
      return;
    }

    final Color color = computeColor();
    for (int i = 0; i < elements.length; i++)
    {
      elements[i].getStyle().setStyleProperty(ElementStyleSheet.PAINT, color);
    }
  }

  private Color computeColor ()
  {
    final Object o = getDataRow().get(field);
    if (o instanceof Number == false)
    {
      return defaultColor;
    }

    Number n = (Number) o;
    Double d;
    if (useAbsoluteValue)
    {
      d = new Double(Math.abs(n.doubleValue()));
    }
    else
    {
      d = new Double(n.doubleValue());
    }

    final int size = Math.min (values.size(), colors.size());
    if (useOppositeLogic)
    {
      for (int i = size - 1; i >= 0; i++)
      {
        Double c = (Double) values.get(i);
        // limit value is greater than found value
        if (c.compareTo(d) > 0)
        {
          return (Color) colors.get(i);
        }
      }
    }
    else
    {
      for (int i = 0; i < size; i++)
      {
        Double c = (Double) values.get(i);
        // limit value is lower than found value
        if (c.compareTo(d) < 0)
        {
          return (Color) colors.get(i);
        }
      }
    }
    return defaultColor;
  }
  /**
   * This method does not return anything.
   * 
   * @see org.jfree.report.function.Expression#getValue()
   */
  public Object getValue()
  {
    return null;
  }

  /**
   * Defines, whether all negative values should be made positive, by calling
   * 'Math.abs'.
   *
   * @return Returns the useAbsoluteValue.
   */
  public boolean isUseAbsoluteValue()
  {
    return useAbsoluteValue;
  }

  /**
   * Defines, whether all negative values should be made positive, by calling
   * 'Math.abs'.
   *
   * @param useAbsoluteValue The useAbsoluteValue to set. 
   */
  public void setUseAbsoluteValue(boolean useAbsoluteValue)
  {
    this.useAbsoluteValue = useAbsoluteValue;
  }

  public boolean isUseOppositeLogic()
  {
    return useOppositeLogic;
  }

  public void setUseOppositeLogic(final boolean useOppositeLogic)
  {
    this.useOppositeLogic = useOppositeLogic;
  }

  public void setColor (final int index, final Color color)
  {
    if (colors.size() == index)
    {
      colors.add(color);
    }
    else
    {
      colors.set(index, color);
    }
  }

  public Color getColor (final int index)
  {
    return (Color) colors.get(index);
  }

  public int getColorCount ()
  {
    return colors.size();
  }

  public Color[] getColor ()
  {
    return (Color[]) colors.toArray(new String[colors.size()]);
  }

  public void setColor (final Color[] colors)
  {
    this.colors.clear();
    this.colors.addAll(Arrays.asList(colors));
  }

  public void setLimit (final int index, final Double value)
  {
    if (colors.size() == index)
    {
      values.add(value);
    }
    else
    {
      values.set(index, value);
    }
  }

  public Double getLimit (final int index)
  {
    return (Double) values.get(index);
  }

  public int getLimitCount ()
  {
    return values.size();
  }

  public Double[] getLimit ()
  {
    return (Double[]) values.toArray(new Double[values.size()]);
  }

  public void setLimit (final Double[] limits)
  {
    this.values.clear();
    this.values.addAll(Arrays.asList(limits));
  }

  /**
   * Clones the function. <P> Be aware, this does not create a deep copy. If you
   * have complex strucures contained in objects, you have to override this
   * function.
   *
   * @return a clone of this function.
   * @throws CloneNotSupportedException this should never happen.
   */
  public Object clone() throws CloneNotSupportedException
  {
    ElementTrafficLightFunction elf = (ElementTrafficLightFunction) super.clone();
    elf.values = (ArrayList) values.clone();
    elf.colors = (ArrayList) colors.clone();
    return elf;
  }

}
