/**
 * ========================================
 * JFreeReport : a free Java report library
 * ========================================
 *
 * Project Info:  http://www.jfree.org/jfreereport/index.html
 * Project Lead:  Thomas Morgner;
 *
 * (C) Copyright 2000-2003, by Simba Management Limited and Contributors.
 *
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation;
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * ------------------------------
 * CSVExportPlugin.java
 * ------------------------------
 * (C)opyright 2003, by Thomas Morgner and Contributors.
 *
 * Original Author:  Thomas Morgner;
 * Contributor(s):   David Gilbert (for Simba Management Limited);
 *
 * $Id: CSVExportPlugin.java,v 1.21 2006/02/10 14:58:44 taqua Exp $
 *
 * Changes
 * -------------------------
 * 13-Jun-2003 : Initial version
 *
 */

package org.jfree.report.modules.gui.csv;

import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Window;
import javax.swing.Icon;
import javax.swing.KeyStroke;

import org.jfree.report.JFreeReport;
import org.jfree.report.JFreeReportBoot;
import org.jfree.report.modules.gui.base.AbstractExportPlugin;
import org.jfree.report.modules.gui.base.ExportTask;
import org.jfree.report.modules.gui.base.PreviewProxy;
import org.jfree.report.modules.gui.base.PreviewProxyBase;
import org.jfree.report.modules.gui.base.ReportProgressDialog;
import org.jfree.ui.RefineryUtilities;
import org.jfree.util.ResourceBundleSupport;

/**
 * Encapsulates the CSVExportDialog into a separate plugin.
 *
 * @author Thomas Morgner
 */
public class CSVExportPlugin extends AbstractExportPlugin
{
  /**
   * The CSV export dialog.
   */
  private CSVExportDialog exportDialog;

  /**
   * Localised resources.
   */
  private final ResourceBundleSupport resources;

  /**
   * The base resource class.
   */
  public static final String BASE_RESOURCE_CLASS =
          "org.jfree.report.modules.gui.csv.resources.csv-export-resources";
  public static final String PROGRESS_DIALOG_ENABLE_KEY =
          "org.jfree.report.modules.gui.csv.ProgressDialogEnabled";

  /**
   * DefaultConstructor.
   */
  public CSVExportPlugin ()
  {
    resources = new ResourceBundleSupport(BASE_RESOURCE_CLASS);
  }

  /**
   * Creates the report progress dialog used to monitor the export.
   *
   * @return the created dialog.
   */
  protected ReportProgressDialog createProgressDialog ()
  {
    final ReportProgressDialog progressDialog = super.createProgressDialog();
    progressDialog.setDefaultCloseOperation(ReportProgressDialog.DO_NOTHING_ON_CLOSE);
    progressDialog.setTitle(resources.getString("cvs-export.progressdialog.title"));
    progressDialog.setMessage(resources.getString("cvs-export.progressdialog.message"));
    progressDialog.pack();
    RefineryUtilities.positionFrameRandomly(progressDialog);
    return progressDialog;
  }

  /**
   * Shows this dialog and (if the dialog is confirmed) saves the complete report into an
   * comma separated values file.
   *
   * @param report the report being processed.
   * @return true or false.
   */
  public boolean performExport (final JFreeReport report)
  {
    final CSVExportDialog exportDialog = getExportDialog();
    final boolean result = exportDialog.performQueryForExport(report);
    if (result == false)
    {
      // user canceled the dialog ...
      return handleExportResult(true);
    }

    final ExportTask task;
    if (exportDialog.isExportRawData())
    {
      task = new CSVRawExportTask
              (exportDialog.getFilename(), exportDialog.getEncoding(), report);
    }
    else
    {
      ReportProgressDialog progressDialog = null;
      if (report.getReportConfiguration().getConfigProperty
              (PROGRESS_DIALOG_ENABLE_KEY,
                      "false").equals("true"))
      {
        progressDialog = createProgressDialog();
      }

      task = new CSVTableExportTask
              (exportDialog.getFilename(), exportDialog.getEncoding(),
                      progressDialog, report);
    }
    task.addExportTaskListener(new DefaultExportTaskListener());
    delegateTask(task);
    return handleExportResult(task);
  }

  /**
   * Returns the dialog used to query the export parameters from the user.
   *
   * @return the export dialog.
   */
  protected synchronized CSVExportDialog getExportDialog ()
  {
    if (exportDialog == null)
    {
      final PreviewProxy proxy = super.getProxy();
      if (proxy instanceof Frame)
      {
        exportDialog = new CSVExportDialog((Frame) proxy);
      }
      else if (proxy instanceof Dialog)
      {
        exportDialog = new CSVExportDialog((Dialog) proxy);
      }
      else
      {
        // look where we have been added ...
        if (proxy != null)
        {
          final PreviewProxyBase base = proxy.getBase();
          final Window w = PreviewProxyBase.getWindowAncestor(base);
          if (w instanceof Frame)
          {
            exportDialog = new CSVExportDialog((Frame) w);
          }
          else if (w instanceof Dialog)
          {
            exportDialog = new CSVExportDialog((Dialog) w);
          }
          else
          {
            exportDialog = new CSVExportDialog();
          }
        }
        else
        {
          exportDialog = new CSVExportDialog();
        }
      }
      exportDialog.pack();
    }
    return exportDialog;
  }

  /**
   * Returns the resourcebundle to be used to translate strings into localized content.
   *
   * @return the resourcebundle for the localisation.
   */
  protected ResourceBundleSupport getResources ()
  {
    return resources;
  }

  /**
   * Returns the display name for the CSV dialog.
   *
   * @return The name.
   */
  public String getDisplayName ()
  {
    return resources.getString("action.export-to-csv.name");
  }

  /**
   * Returns a short description for the CSV dialog.
   *
   * @return The description.
   */
  public String getShortDescription ()
  {
    return resources.getString("action.export-to-csv.description");
  }

  /**
   * Returns the small icon for the dialog.
   *
   * @return The icon.
   */
  public Icon getSmallIcon ()
  {
    return getSkin().getIcon("action.export-to-csv.small-icon", true, false);
  }

  /**
   * Returns the large icon for the dialog.
   *
   * @return The icon.
   */
  public Icon getLargeIcon ()
  {
    return getSkin().getIcon("action.export-to-csv.icon", true, true);
  }

  /**
   * Returns the accelerator key for the action associated with the dialog.
   *
   * @return The key stroke.
   */
  public KeyStroke getAcceleratorKey ()
  {
    return resources.getKeyStroke("action.export-to-csv.accelerator");
  }

  /**
   * Returns the mnemonic key code for the action associated with the dialog.
   *
   * @return The key code.
   */
  public Integer getMnemonicKey ()
  {
    return resources.getMnemonic("action.export-to-csv.mnemonic");
  }


  /**
   * Returns true if the action should be added to the toolbar, and false otherwise.
   *
   * @return true, if the plugin should be added to the toolbar, false otherwise.
   */
  public boolean isAddToToolbar ()
  {
    return JFreeReportBoot.getInstance().getGlobalConfig().getConfigProperty
            ("org.jfree.report.modules.gui.csv.AddToToolbar", "false").equals("true");
  }

  /**
   * Returns true if the action is separated, and false otherwise. A separated action
   * starts a new action group and will be spearated from previous actions on the menu and
   * toolbar.
   *
   * @return true, if the action should be separated from previous actions, false
   *         otherwise.
   */
  public boolean isSeparated ()
  {
    return JFreeReportBoot.getInstance().getGlobalConfig().getConfigProperty
            ("org.jfree.report.modules.gui.csv.Separated", "false").equals("true");
  }


}
