/**
 * ========================================
 * JFreeReport : a free Java report library
 * ========================================
 *
 * Project Info:  http://www.jfree.org/jfreereport/index.html
 * Project Lead:  Thomas Morgner;
 *
 * (C) Copyright 2000-2003, by Simba Management Limited and Contributors.
 *
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation;
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * ------------------
 * PDFSaveDialog.java
 * ------------------
 * (C)opyright 2002, 2003, by Thomas Morgner and Contributors.
 *
 * Original Author:  Thomas Morgner;
 * Contributor(s):   David Gilbert (for Simba Management Limited);
 *
 * $Id: PDFSaveDialog.java,v 1.24.2.1 2006/05/27 16:58:27 taqua Exp $
 *
 * Changes
 * --------
 * 26-Aug-2002 : Initial version
 * 29-Aug-2002 : Downport to JDK 1.2.2
 * 31-Aug-2002 : Documentation
 * 01-Sep-2002 : Printing security was inaccurate; bug (exception in JOptionPane) when file exists
 *               More documentation
 * 10-Dec-2002 : Minor Javadoc changes (DG);
 *
 */

package org.jfree.report.modules.gui.pdf;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.text.MessageFormat;
import java.util.Properties;
import java.util.ResourceBundle;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.KeyStroke;

import org.jfree.base.config.ModifiableConfiguration;
import org.jfree.report.JFreeReport;
import org.jfree.report.JFreeReportBoot;
import org.jfree.report.modules.gui.base.components.AbstractExportDialog;
import org.jfree.report.modules.gui.base.components.EncodingComboBoxModel;
import org.jfree.report.modules.gui.base.components.JStatusBar;
import org.jfree.report.modules.output.pageable.pdf.PDFOutputTarget;
import org.jfree.report.util.EncodingSupport;
import org.jfree.report.util.StringUtil;
import org.jfree.ui.FilesystemFilter;
import org.jfree.ui.action.ActionButton;
import org.jfree.util.Configuration;
import org.jfree.util.Log;

/**
 * A dialog that is used to perform the printing of a report into a PDF file. It
 * is primarily used to edit the properties of the {@link PDFOutputTarget}
 * before the target is used to print the report.
 * <p/>
 * The main method to call the dialog is PDFSaveDialog.savePDF(). Given a report
 * and a pageformat, the dialog is shown and if the user approved the dialog,
 * the pdf is saved using the settings made in the dialog.
 *
 * @author Thomas Morgner
 */
public class PDFSaveDialog extends AbstractExportDialog
{
  /** Useful constant. */
  private static final int CBMODEL_NOPRINTING = 0;

  /** Useful constant. */
  private static final int CBMODEL_DEGRADED = 1;

  /** Useful constant. */
  private static final int CBMODEL_FULL = 2;

  /**
   * Internal action class to enable/disable the Security-Settings panel.
   * Without encryption a pdf file cannot have any security settings enabled.
   */
  private class ActionSecuritySelection extends AbstractAction
  {
    /** Default constructor. */
    public ActionSecuritySelection()
    {
    }

    /**
     * Receives notification that the action has occurred.
     *
     * @param e the action event.
     */
    public void actionPerformed(final ActionEvent e)
    {
      updateSecurityPanelEnabled();
    }
  }

  /** Internal action class to confirm the dialog and to validate the input. */
  private class ActionConfirm extends AbstractConfirmAction
  {
    /** Default constructor. */
    public ActionConfirm(final ResourceBundle resources)
    {
      putValue(Action.NAME, resources.getString("pdfsavedialog.confirm"));
    }
  }

  /** Internal action class to cancel the report processing. */
  private class ActionCancel extends AbstractCancelAction
  {
    /** Default constructor. */
    public ActionCancel(final ResourceBundle resources)
    {
      putValue(Action.NAME, resources.getString("pdfsavedialog.cancel"));
    }
  }

  /** Internal action class to select a target file. */
  private class ActionSelectFile extends AbstractAction
  {
    /** Default constructor. */
    public ActionSelectFile(final ResourceBundle resources)
    {
      putValue(Action.NAME, resources.getString("pdfsavedialog.selectFile"));
    }

    /**
     * Receives notification that the action has occurred.
     *
     * @param e the action event.
     */
    public void actionPerformed(final ActionEvent e)
    {
      performSelectFile();
    }
  }

  /** Security selection action. */
  private Action actionSecuritySelection;

  /** Select file action. */
  private Action actionSelectFile;

  /** Filename text field. */
  private JTextField txFilename;

  /** Author text field. */
  private JTextField txAuthor;

  /** Title text field. */
  private JTextField txTitle;

  /** Security (none) radio button. */
  private JRadioButton rbSecurityNone;

  /** Security (40 bit) radio button. */
  private JRadioButton rbSecurity40Bit;

  /** Security (128 bit) radio button. */
  private JRadioButton rbSecurity128Bit;

  /** User password text field. */
  private JTextField txUserPassword;

  /** Owner password text field. */
  private JTextField txOwnerPassword;

  /** Confirm user password text field. */
  private JTextField txConfUserPassword;

  /** Confirm ownder password text field. */
  private JTextField txConfOwnerPassword;

  /** Allow copy check box. */
  private JCheckBox cxAllowCopy;

  /** Allow screen readers check box. */
  private JCheckBox cxAllowScreenReaders;

  /** Allow printing check box. */
  private JComboBox cbAllowPrinting;

  /** Allow assembly check box. */
  private JCheckBox cxAllowAssembly;

  /** Allow modify contents check box. */
  private JCheckBox cxAllowModifyContents;

  /** Allow modify annotations check box. */
  private JCheckBox cxAllowModifyAnnotations;

  /** Allow fill in check box. */
  private JCheckBox cxAllowFillIn;

  /** Combo box for selecting the printing model. */
  private DefaultComboBoxModel printingModel;

  /** A combo-box for selecting the encoding. */
  private JComboBox cbEncoding;

  /** A model for the available encodings. */
  private EncodingComboBoxModel encodingModel;

  /** A file chooser. */
  private JFileChooser fileChooser;

  /**
   * Creates a new PDF save dialog.
   *
   * @param owner the dialog owner.
   */
  public PDFSaveDialog(final Frame owner)
  {
    super(owner);
    initConstructor();
  }

  /**
   * Creates a new PDF save dialog.
   *
   * @param owner the dialog owner.
   */
  public PDFSaveDialog(final Dialog owner)
  {
    super(owner);
    initConstructor();
  }

  /** Creates a new PDF save dialog.  The created dialog is modal. */
  public PDFSaveDialog()
  {
    initConstructor();
  }

  /**
   * Updates the security panel state. If no encryption is selected, all
   * security setting components will be disabled.
   */
  protected void updateSecurityPanelEnabled()
  {
    final boolean b = (rbSecurityNone.isSelected() == false);
    txUserPassword.setEnabled(b);
    txOwnerPassword.setEnabled(b);
    txConfOwnerPassword.setEnabled(b);
    txConfUserPassword.setEnabled(b);
    cxAllowAssembly.setEnabled(b);
    cxAllowCopy.setEnabled(b);
    cbAllowPrinting.setEnabled(b);
    cxAllowFillIn.setEnabled(b);
    cxAllowModifyAnnotations.setEnabled(b);
    cxAllowModifyContents.setEnabled(b);
    cxAllowScreenReaders.setEnabled(b);
  }

  /** Initialisation. */
  private void initConstructor()
  {
    setCancelAction(new ActionCancel(getResources()));
    setConfirmAction(new ActionConfirm(getResources()));
    actionSelectFile = new ActionSelectFile(getResources());

    setTitle(getResources().getString("pdfsavedialog.dialogtitle"));
    initialize();
    clear();
  }

  protected String getConfigurationSuffix()
  {
    return "_pdfexport";
  }

  protected String getResourceBaseName()
  {
    return PDFExportPlugin.BASE_RESOURCE_CLASS;
  }

  /**
   * Gets and initializes the the combobox model for the security setting
   * "allowPrinting".
   *
   * @return the combobox model containing the different values for the
   *         allowPrinting option.
   */
  private DefaultComboBoxModel getPrintingComboBoxModel()
  {
    if (printingModel == null)
    {
      final Object[] data = {
              getResources().getString("pdfsavedialog.option.noprinting"),
              getResources().getString("pdfsavedialog.option.degradedprinting"),
              getResources().getString("pdfsavedialog.option.fullprinting")
      };
      printingModel = new DefaultComboBoxModel(data);
    }
    return printingModel;
  }

  /**
   * Returns a single instance of the security selection action.
   *
   * @return the action.
   */
  private Action getActionSecuritySelection()
  {
    if (actionSecuritySelection == null)
    {
      actionSecuritySelection = new ActionSecuritySelection();
    }
    return actionSecuritySelection;
  }

  /**
   * Returns a single instance of the file selection action.
   *
   * @return the action.
   */
  private Action getActionSelectFile()
  {
    return actionSelectFile;
  }

  /** Initializes the Swing components of this dialog. */
  private void initialize()
  {
    final JPanel contentPane = new JPanel();
    contentPane.setLayout(new GridBagLayout());
    contentPane.setBorder(BorderFactory.createEmptyBorder(3, 3, 3, 3));

    final JLabel lblFileName = new JLabel(getResources().getString(
            "pdfsavedialog.filename"));
    final JLabel lblAuthor = new JLabel(getResources().getString(
            "pdfsavedialog.author"));
    final JLabel lblTitel = new JLabel(getResources().getString(
            "pdfsavedialog.title"));
    final JLabel lblEncoding = new JLabel(getResources().getString(
            "pdfsavedialog.encoding"));
    final JButton btnSelect = new ActionButton(getActionSelectFile());

    txAuthor = new JTextField();
    txFilename = new JTextField();
    txTitle = new JTextField();
    encodingModel = EncodingComboBoxModel.createDefaultModel();
    encodingModel.addEncodingUnchecked("Identity-H", "PDF-Unicode encoding");
    encodingModel.addEncodingUnchecked("Identity-V", "PDF-Unicode encoding");
    encodingModel.sort();
    cbEncoding = new JComboBox(encodingModel);

    GridBagConstraints gbc = new GridBagConstraints();
    gbc.gridx = 0;
    gbc.gridy = 0;
    gbc.anchor = GridBagConstraints.WEST;
    gbc.insets = new Insets(3, 1, 1, 1);
    contentPane.add(lblFileName, gbc);

    gbc = new GridBagConstraints();
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridx = 0;
    gbc.gridy = 1;
    gbc.insets = new Insets(1, 1, 1, 1);
    contentPane.add(lblTitel, gbc);

    gbc = new GridBagConstraints();
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridx = 0;
    gbc.gridy = 2;
    gbc.insets = new Insets(1, 1, 1, 1);
    contentPane.add(lblAuthor, gbc);

    gbc = new GridBagConstraints();
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridx = 0;
    gbc.gridy = 3;
    gbc.insets = new Insets(1, 1, 1, 1);
    contentPane.add(lblEncoding, gbc);

    gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.weightx = 1;
    gbc.gridx = 1;
    gbc.gridy = 0;
    gbc.ipadx = 120;
    gbc.insets = new Insets(3, 1, 1, 1);
    contentPane.add(txFilename, gbc);

    gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.weightx = 1;
    gbc.gridx = 1;
    gbc.gridy = 1;
    gbc.ipadx = 120;
    gbc.insets = new Insets(1, 1, 1, 1);
    contentPane.add(txTitle, gbc);

    gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.weightx = 1;
    gbc.gridx = 1;
    gbc.gridy = 2;
    gbc.ipadx = 120;
    gbc.insets = new Insets(1, 1, 1, 1);
    contentPane.add(txAuthor, gbc);

    gbc = new GridBagConstraints();
    gbc.anchor = GridBagConstraints.NORTHWEST;
    gbc.gridx = 2;
    gbc.gridy = 0;
    gbc.gridheight = 2;
    contentPane.add(btnSelect, gbc);

    gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.weightx = 1;
    gbc.gridx = 1;
    gbc.gridy = 3;
    gbc.ipadx = 120;
    gbc.insets = new Insets(1, 1, 1, 1);
    contentPane.add(cbEncoding, gbc);

    gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.weightx = 1;
    gbc.weighty = 1;
    gbc.gridx = 0;
    gbc.gridwidth = 3;
    gbc.gridy = 4;
    gbc.insets = new Insets(15, 0, 0, 0);
    gbc.anchor = GridBagConstraints.NORTH;
    contentPane.add(createSecurityPanel(), gbc);

    final JButton btnCancel = new ActionButton(getCancelAction());
    final JButton btnConfirm = new ActionButton(getConfirmAction());
    final JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new GridLayout(1, 2, 5, 5));
    buttonPanel.add(btnConfirm);
    buttonPanel.add(btnCancel);
    btnConfirm.setDefaultCapable(true);
    getRootPane().setDefaultButton(btnConfirm);
    buttonPanel.registerKeyboardAction(getConfirmAction(),
            KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),
            JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

    gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.NONE;
    gbc.anchor = GridBagConstraints.EAST;
    gbc.weightx = 1;
    gbc.gridx = 0;
    gbc.gridwidth = 3;
    gbc.gridy = 6;
    gbc.insets = new Insets(10, 0, 0, 0);
    contentPane.add(buttonPanel, gbc);

    final JPanel contentWithStatus = new JPanel();
    contentWithStatus.setLayout(new BorderLayout());
    contentWithStatus.add(contentPane, BorderLayout.CENTER);
    contentWithStatus.add(getStatusBar(), BorderLayout.SOUTH);

    setContentPane(contentWithStatus);

    getFormValidator().registerTextField(txFilename);
    getFormValidator().registerTextField(txConfOwnerPassword);
    getFormValidator().registerTextField(txConfUserPassword);
    getFormValidator().registerTextField(txUserPassword);
    getFormValidator().registerTextField(txOwnerPassword);
  }

  /** Initializes the class member components of the security panel. */
  private void createSecurityPanelComponents()
  {
    txUserPassword = new JPasswordField();
    txConfUserPassword = new JPasswordField();
    txOwnerPassword = new JPasswordField();
    txConfOwnerPassword = new JPasswordField();

    cxAllowCopy = new JCheckBox(getResources().getString(
            "pdfsavedialog.allowCopy"));
    cbAllowPrinting = new JComboBox(getPrintingComboBoxModel());
    cxAllowScreenReaders =
            new JCheckBox(getResources().getString(
                    "pdfsavedialog.allowScreenreader"));

    cxAllowAssembly = new JCheckBox(getResources().getString(
            "pdfsavedialog.allowAssembly"));
    cxAllowModifyContents =
            new JCheckBox(getResources().getString(
                    "pdfsavedialog.allowModifyContents"));
    cxAllowModifyAnnotations =
            new JCheckBox(getResources().getString(
                    "pdfsavedialog.allowModifyAnnotations"));
    cxAllowFillIn = new JCheckBox(getResources().getString(
            "pdfsavedialog.allowFillIn"));

  }

  /**
   * Creates a panel for the security settings.
   *
   * @return The panel.
   */
  private JPanel createSecurityPanel()
  {
    final JPanel securityPanel = new JPanel();
    securityPanel.setLayout(new GridBagLayout());
    securityPanel.setBorder(BorderFactory.createTitledBorder(
            getResources().getString("pdfsavedialog.security")));

    createSecurityPanelComponents();

    final JLabel lblUserPass = new JLabel(getResources().getString(
            "pdfsavedialog.userpassword"));
    final JLabel lblUserPassConfirm =
            new JLabel(getResources().getString(
                    "pdfsavedialog.userpasswordconfirm"));
    final JLabel lblOwnerPass =
            new JLabel(getResources().getString("pdfsavedialog.ownerpassword"));
    final JLabel lblOwnerPassConfirm =
            new JLabel(getResources().getString(
                    "pdfsavedialog.ownerpasswordconfirm"));
    final JLabel lbAllowPrinting =
            new JLabel(getResources().getString("pdfsavedialog.allowPrinting"));

    GridBagConstraints gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.weightx = 1;
    gbc.gridx = 0;
    gbc.gridwidth = 4;
    gbc.gridy = 0;
    gbc.insets = new Insets(5, 5, 5, 5);
    securityPanel.add(createSecurityConfigPanel(), gbc);

    gbc = new GridBagConstraints();
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridx = 0;
    gbc.gridy = 1;
    gbc.insets = new Insets(5, 5, 5, 5);
    securityPanel.add(lblUserPass, gbc);

    gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.weightx = 1;
    gbc.gridx = 1;
    gbc.gridy = 1;
    gbc.ipadx = 120;
    gbc.insets = new Insets(5, 5, 5, 5);
    securityPanel.add(txUserPassword, gbc);

    gbc = new GridBagConstraints();
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridx = 0;
    gbc.gridy = 2;
    gbc.insets = new Insets(5, 5, 5, 5);
    securityPanel.add(lblOwnerPass, gbc);

    gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.weightx = 1;
    gbc.gridx = 1;
    gbc.gridy = 2;
    gbc.ipadx = 120;
    gbc.insets = new Insets(5, 5, 5, 5);
    securityPanel.add(txOwnerPassword, gbc);

    gbc = new GridBagConstraints();
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridx = 2;
    gbc.gridy = 1;
    gbc.insets = new Insets(5, 5, 5, 5);
    securityPanel.add(lblUserPassConfirm, gbc);

    gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.weightx = 1;
    gbc.gridx = 3;
    gbc.gridy = 1;
    gbc.ipadx = 120;
    gbc.insets = new Insets(5, 5, 5, 5);
    securityPanel.add(txConfUserPassword, gbc);

    gbc = new GridBagConstraints();
    gbc.anchor = GridBagConstraints.WEST;
    gbc.gridx = 2;
    gbc.gridy = 2;
    gbc.insets = new Insets(5, 5, 5, 5);
    securityPanel.add(lblOwnerPassConfirm, gbc);

    gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.weightx = 1;
    gbc.gridx = 3;
    gbc.gridy = 2;
    gbc.ipadx = 120;
    gbc.insets = new Insets(5, 5, 5, 5);
    securityPanel.add(txConfOwnerPassword, gbc);

    gbc = new GridBagConstraints();
    gbc.gridx = 0;
    gbc.gridwidth = 2;
    gbc.gridy = 3;
    gbc.anchor = GridBagConstraints.WEST;
    securityPanel.add(cxAllowCopy, gbc);

    gbc = new GridBagConstraints();
    gbc.gridx = 0;
    gbc.gridwidth = 2;
    gbc.gridy = 4;
    gbc.anchor = GridBagConstraints.WEST;
    securityPanel.add(cxAllowScreenReaders, gbc);

    gbc = new GridBagConstraints();
    gbc.gridx = 0;
    gbc.gridwidth = 2;
    gbc.gridy = 5;
    gbc.anchor = GridBagConstraints.WEST;
    securityPanel.add(cxAllowFillIn, gbc);

    gbc = new GridBagConstraints();
    gbc.gridx = 2;
    gbc.gridwidth = 2;
    gbc.gridy = 3;
    gbc.anchor = GridBagConstraints.WEST;
    securityPanel.add(cxAllowAssembly, gbc);

    gbc = new GridBagConstraints();
    gbc.gridx = 2;
    gbc.gridwidth = 2;
    gbc.gridy = 4;
    gbc.anchor = GridBagConstraints.WEST;
    securityPanel.add(cxAllowModifyContents, gbc);

    gbc = new GridBagConstraints();
    gbc.gridx = 2;
    gbc.gridwidth = 2;
    gbc.gridy = 5;
    gbc.anchor = GridBagConstraints.WEST;
    securityPanel.add(cxAllowModifyAnnotations, gbc);

    gbc = new GridBagConstraints();
    gbc.gridx = 0;
    gbc.gridwidth = 1;
    gbc.gridy = 6;
    gbc.anchor = GridBagConstraints.WEST;
    securityPanel.add(lbAllowPrinting, gbc);

    gbc = new GridBagConstraints();
    gbc.gridx = 1;
    gbc.gridwidth = 3;
    gbc.gridy = 6;
    gbc.anchor = GridBagConstraints.WEST;
    securityPanel.add(cbAllowPrinting, gbc);

    return securityPanel;
  }

  /**
   * Creates the security config panel. This panel is used to select the level
   * of the PDF security.
   *
   * @return the created security config panel.
   */
  private JPanel createSecurityConfigPanel()
  {
    rbSecurityNone = new JRadioButton(getResources().getString(
            "pdfsavedialog.securityNone"));
    rbSecurity40Bit = new JRadioButton(getResources().getString(
            "pdfsavedialog.security40bit"));
    rbSecurity128Bit = new JRadioButton(getResources().getString(
            "pdfsavedialog.security128bit"));

    rbSecurityNone.addActionListener(getActionSecuritySelection());
    rbSecurity40Bit.addActionListener(getActionSecuritySelection());
    rbSecurity128Bit.addActionListener(getActionSecuritySelection());

    rbSecurity128Bit.setSelected(true);

    final JPanel pnlSecurityConfig = new JPanel();
    pnlSecurityConfig.setLayout(new GridLayout());
    pnlSecurityConfig.add(rbSecurityNone);
    pnlSecurityConfig.add(rbSecurity40Bit);
    pnlSecurityConfig.add(rbSecurity128Bit);

    final ButtonGroup btGrpSecurity = new ButtonGroup();
    btGrpSecurity.add(rbSecurity128Bit);
    btGrpSecurity.add(rbSecurity40Bit);
    btGrpSecurity.add(rbSecurityNone);

    return pnlSecurityConfig;
  }

  /**
   * returns the defined user password for the pdf file. The user password
   * limits read-only access to the pdf in the PDF-Viewer. The reader/user has
   * to enter the password when opening the file.
   *
   * @return the defined password. The password can be null.
   */
  public String getUserPassword()
  {
    final String txt = txUserPassword.getText();
    if (txt.equals(""))
    {
      return null;
    }
    return txt;
  }

  /**
   * Defines the user password for the pdf file. The user password limits
   * read-only access to the pdf in the PDF-Viewer. The reader/user has to enter
   * the password when opening the file.
   *
   * @param userPassword the defined password. The password can be null.
   */
  public void setUserPassword(final String userPassword)
  {
    txUserPassword.setText(userPassword);
    txConfUserPassword.setText(userPassword);
  }

  /**
   * Returns the owner password for the pdf file. The owner password limits
   * writing access to the pdf in the PDF-Editor. The user has to enter the
   * password when opening the file to enable editing functionality or to modify
   * the file.
   *
   * @return the defined password. The password can be null.
   */
  public String getOwnerPassword()
  {
    final String txt = txOwnerPassword.getText();
    if (txt.equals(""))
    {
      return null;
    }
    return txt;
  }

  /**
   * Defines the owner password for the pdf file. The owner password limits
   * writing access to the pdf in the PDF-Editor. The user has to enter the
   * password when opening the file to enable editing functionality or to modify
   * the file.
   *
   * @param ownerPassword the defined password. The password can be null.
   */
  public void setOwnerPassword(final String ownerPassword)
  {
    txOwnerPassword.setText(ownerPassword);
    txConfOwnerPassword.setText(ownerPassword);
  }

  /**
   * Returns the selected encoding.
   *
   * @return The encoding.
   */
  public String getEncoding()
  {
    if (cbEncoding.getSelectedIndex() == -1)
    {
      return EncodingSupport.getPlatformDefaultEncoding();
    }
    else
    {
      final String encoding = encodingModel.getEncoding(
              cbEncoding.getSelectedIndex());
      if ("UTF-16".equals(encoding))
      {
        return "Identity-H";
      }
      if ("UTF-8".equals(encoding))
      {
        return "Identity-H";
      }
      return encoding;
    }
  }

  /**
   * Sets the encoding.
   *
   * @param encoding the encoding (name).
   */
  public void setEncoding(final String encoding)
  {
    cbEncoding.setSelectedIndex(encodingModel.indexOf(encoding));
  }

  /**
   * Gets the AllowDegradedPrinting permission for the generated PDF-file.
   *
   * @return true if a low quality printing is allowed, false otherwise.
   */
  public boolean isAllowDegradedPrinting()
  {
    return cbAllowPrinting.getSelectedIndex() == CBMODEL_DEGRADED;
  }

  /**
   * Gets the AllowAssembly permission for the generated PDF-file.
   *
   * @return true, if the generated pdf may be reassembled using an pdf editor.
   */
  public boolean isAllowAssembly()
  {
    return cxAllowAssembly.isSelected();
  }

  /**
   * Defines whether the generated pdf may be reassembled.
   *
   * @param allowAssembly the flag.
   */
  public void setAllowAssembly(final boolean allowAssembly)
  {
    this.cxAllowAssembly.setSelected(allowAssembly);
  }

  /**
   * Defines whether the generated pdf may accessed using screenreaders.
   * Screenreaders are used to make pdf files accessible for disabled people.
   *
   * @return true, if screenreaders are allowed for accessing this file
   */
  public boolean isAllowScreenreaders()
  {
    return cxAllowScreenReaders.isSelected();
  }

  /**
   * Defines whether the generated pdf may accessed using screenreaders.
   * Screenreaders are used to make pdf files accessible for disabled people.
   *
   * @param allowScreenreaders a flag containing true, if screenreaders are
   *                           allowed to access the content of this file, false
   *                           otherwise
   */
  public void setAllowScreenreaders(final boolean allowScreenreaders)
  {
    this.cxAllowScreenReaders.setSelected(allowScreenreaders);
  }

  /**
   * Defines whether the contents of form fields may be changed by the user.
   *
   * @return true, if the user may change the contents of formulars.
   */
  public boolean isAllowFillIn()
  {
    return cxAllowFillIn.isSelected();
  }

  /**
   * Defines whether the contents of form fields may be changed by the user.
   *
   * @param allowFillIn set to true to allow the change/filling of form fields
   */
  public void setAllowFillIn(final boolean allowFillIn)
  {
    this.cxAllowFillIn.setSelected(allowFillIn);
  }

  /**
   * Defines whether the contents of the file are allowed to be copied.
   *
   * @return true, if the user is allowed to copy the contents of the file.
   */
  public boolean isAllowCopy()
  {
    return cxAllowCopy.isSelected();
  }

  /**
   * Defines whether the contents of the file are allowed to be copied.
   *
   * @param allowCopy set to true, if the user is allowed to copy the contents
   *                  of the file.
   */
  public void setAllowCopy(final boolean allowCopy)
  {
    this.cxAllowCopy.setSelected(allowCopy);
  }

  /**
   * Defines whether the user is allowed to add or modify annotations in this
   * file.
   *
   * @return true, if this files annotations can be modified, false otherwise
   */
  public boolean isAllowModifyAnnotations()
  {
    return cxAllowModifyAnnotations.isSelected();
  }

  /**
   * Defines whether the user is allowed to add or modify annotations in this
   * file.
   *
   * @param allowModifyAnnotations the flag.
   */
  public void setAllowModifyAnnotations(final boolean allowModifyAnnotations)
  {
    this.cxAllowModifyAnnotations.setSelected(allowModifyAnnotations);
  }

  /**
   * Defines whether the user is allowed to modify the contents of this file.
   *
   * @return true, if this files content can be modified, false otherwise
   */
  public boolean isAllowModifyContents()
  {
    return cxAllowModifyContents.isSelected();
  }

  /**
   * Defines whether the user is allowed to modify the contents of this file.
   *
   * @param allowModifyContents set to true, if this files content can be
   *                            modified, false otherwise
   */
  public void setAllowModifyContents(final boolean allowModifyContents)
  {
    this.cxAllowModifyContents.setSelected(allowModifyContents);
  }

  /**
   * Defines whether the user is allowed to print the file. If this right is
   * granted, the user is also able to print a degraded version of the file,
   * regardless of the allowDegradedPrinting property
   *
   * @return true, if this file can be printed, false otherwise
   */
  public boolean isAllowPrinting()
  {
    return cbAllowPrinting.getSelectedIndex() == CBMODEL_FULL;
  }

  /**
   * Defines whether the user is allowed to print the file.  If this right is
   * granted, the user is also able to print a degraded version of the file,
   * regardless of the <code>allowDegradedPrinting</code< property. If you
   * disabled printing but enabled degraded printing, then the user is able to
   * print a low-quality version of the document.
   *
   * @param allowPrinting set to <code>true</code>, if this file can be printed,
   *                      <code>false</code> otherwise.
   * @param degraded      set to <code>true</code>, to allow degraded printing,
   *                      and <code>false</code> otherwise.
   */
  public void setPrintLevel(final boolean allowPrinting, final boolean degraded)
  {
    if (allowPrinting == true)
    {
      this.cbAllowPrinting.setSelectedIndex(CBMODEL_FULL);
    }
    else
    {
      if (degraded)
      {
        this.cbAllowPrinting.setSelectedIndex(CBMODEL_DEGRADED);
      }
      else
      {
        this.cbAllowPrinting.setSelectedIndex(CBMODEL_NOPRINTING);
      }
    }
  }

  /**
   * Defines the filename of the pdf file.
   *
   * @return the name of the file where to save the pdf file.
   */
  public String getFilename()
  {
    return txFilename.getText();
  }

  /**
   * Defines the filename of the pdf file.
   *
   * @param filename the filename of the pdf file
   */
  public void setFilename(final String filename)
  {
    this.txFilename.setText(filename);
  }

  /**
   * Defines the title of the pdf file.
   *
   * @return the title
   */
  public String getPDFTitle()
  {
    return txTitle.getText();
  }

  /**
   * Defines the title of the pdf file.
   *
   * @param title the title
   */
  public void setPDFTitle(final String title)
  {
    this.txTitle.setText(title);
  }

  /**
   * Gets the author of this report.
   *
   * @return the name of the author of this report.
   */
  public String getAuthor()
  {
    return txAuthor.getText();
  }

  /**
   * Defines the Author of the report. Any freeform text is valid. This defaults
   * to the value of the systemProperty "user.name".
   *
   * @param author the name of the author.
   */
  public void setAuthor(final String author)
  {
    this.txAuthor.setText(author);
  }

  /**
   * Queries the currently selected encryption. If an encryption is selected
   * this method returns either Boolean.TRUE or Boolean.FALSE, when no
   * encryption is set, <code>null</code> is returned. If no encryption is set,
   * the security properties have no defined state.
   *
   * @return the selection state for the encryption. If no encryption is set,
   *         this method returns null, if 40-bit encryption is set, the method
   *         returns Boolean.FALSE and on 128-Bit-encryption, Boolean.TRUE is
   *         returned.
   */
  public String getEncryptionValue()
  {
    if (rbSecurity40Bit.isSelected())
    {
      return PDFOutputTarget.SECURITY_ENCRYPTION_40BIT;
    }
    if (rbSecurity128Bit.isSelected())
    {
      return PDFOutputTarget.SECURITY_ENCRYPTION_128BIT;
    }
    return PDFOutputTarget.SECURITY_ENCRYPTION_NONE;
  }

  /**
   * Defines the currently selected encryption.
   *
   * @param b the new encryption state, one of null, Boolean.TRUE or
   *          Boolean.FALSE
   */
  public void setEncryptionValue(final String b)
  {
    if (b != null)
    {
      if (b.equals(PDFOutputTarget.SECURITY_ENCRYPTION_128BIT))
      {
        rbSecurity128Bit.setSelected(true);
        updateSecurityPanelEnabled();
        return;
      }
      else if (b.equals(PDFOutputTarget.SECURITY_ENCRYPTION_40BIT))
      {
        rbSecurity40Bit.setSelected(true);
        updateSecurityPanelEnabled();
        return;
      }
      else if (b.equals(PDFOutputTarget.SECURITY_ENCRYPTION_NONE) == false)
      {
        Log.warn("Invalid encryption value entered. " + b);
      }
    }
    rbSecurityNone.setSelected(true);
    updateSecurityPanelEnabled();
  }

  /**
   * Clears all selections, input fields and set the selected encryption level
   * to none.
   */
  public void clear()
  {
    txAuthor.setText(
            JFreeReportBoot.getInstance().getGlobalConfig().getConfigProperty(
                    "user.name", ""));
    txConfOwnerPassword.setText("");
    txConfUserPassword.setText("");
    txFilename.setText("");
    txOwnerPassword.setText("");
    txTitle.setText("");
    txUserPassword.setText("");

    cxAllowAssembly.setSelected(false);
    cxAllowCopy.setSelected(false);
    cbAllowPrinting.setSelectedIndex(CBMODEL_NOPRINTING);
    cxAllowFillIn.setSelected(false);
    cxAllowModifyAnnotations.setSelected(false);
    cxAllowModifyContents.setSelected(false);
    cxAllowScreenReaders.setSelected(false);

    rbSecurityNone.setSelected(true);
    getActionSecuritySelection().actionPerformed(null);

    cbEncoding.setSelectedIndex
            (encodingModel.indexOf(
                    EncodingSupport.getPlatformDefaultEncoding()));
  }

  /**
   * Returns the user input of this dialog as properties collection.
   *
   * @return the user input.
   */
  public Properties getDialogContents(final boolean full)
  {
    final Properties p = new Properties();
    p.setProperty("author", getAuthor());
    if (full)
    {
      p.setProperty("filename", getFilename());
    }
    p.setProperty("encoding", getEncoding());
    p.setProperty("title", getPDFTitle());
    p.setProperty("encryption", getEncryptionValue());

    p.setProperty("allow-assembly", String.valueOf(isAllowAssembly()));
    p.setProperty("allow-copy", String.valueOf(isAllowCopy()));
    p.setProperty("allow-degraded-printing", String.valueOf(
            isAllowDegradedPrinting()));
    p.setProperty("allow-fill-in", String.valueOf(isAllowFillIn()));
    p.setProperty("allow-modify-annotations", String.valueOf(
            isAllowModifyAnnotations()));
    p.setProperty("allow-modify-contents", String.valueOf(
            isAllowModifyContents()));
    p.setProperty("allow-printing", String.valueOf(isAllowPrinting()));
    p.setProperty("allow-screenreaders", String.valueOf(
            isAllowScreenreaders()));
    return p;
  }

  /**
   * Restores the user input from a properties collection.
   *
   * @param p the user input.
   */
  public void setDialogContents(final Properties p, final boolean full)
  {
    setAuthor(p.getProperty("author", getAuthor()));
    setEncoding(p.getProperty("encoding", getEncoding()));
    if (full)
    {
      setFilename(p.getProperty("filename", getFilename()));
    }
    setPDFTitle(p.getProperty("title", getPDFTitle()));
    setEncryptionValue(p.getProperty("encryption", getEncryptionValue()));

    setAllowAssembly(StringUtil.parseBoolean(p.getProperty("allow-assembly"),
            isAllowAssembly()));
    setAllowCopy(StringUtil.parseBoolean(p.getProperty("allow-copy"),
            isAllowCopy()));
    setAllowFillIn(StringUtil.parseBoolean(p.getProperty("allow-fill-in"),
            isAllowFillIn()));
    setAllowModifyAnnotations(StringUtil.parseBoolean(p.getProperty(
            "allow-modify-annotations"), isAllowModifyAnnotations()));
    setAllowModifyContents(StringUtil.parseBoolean(p.getProperty(
            "allow-modify-contents"), isAllowModifyContents()));
    setPrintLevel(StringUtil.parseBoolean(p.getProperty("allow-printing"),
            isAllowPrinting()),
            StringUtil.parseBoolean(p.getProperty("allow-degraded-printing"),
                    isAllowDegradedPrinting()));
    setAllowScreenreaders(StringUtil.parseBoolean(p.getProperty(
            "allow-screenreaders"), isAllowScreenreaders()));
  }

  /** selects a file to use as target for the report processing. */
  protected void performSelectFile()
  {
    // lazy initialize ... the file chooser is one of the hot spots here ...
    if (fileChooser == null)
    {
      fileChooser = new JFileChooser();
      final FilesystemFilter filter = new FilesystemFilter(".pdf",
              "PDF Documents");
      fileChooser.addChoosableFileFilter(filter);
      fileChooser.setMultiSelectionEnabled(false);
    }

    final File file = new File(getFilename());
    fileChooser.setCurrentDirectory(file);
    fileChooser.setSelectedFile(file);
    final int option = fileChooser.showSaveDialog(this);
    if (option == JFileChooser.APPROVE_OPTION)
    {
      final File selFile = fileChooser.getSelectedFile();
      String selFileName = selFile.getAbsolutePath();

      // Test if ends of pdf
      if (selFileName.toUpperCase().endsWith(".PDF") == false)
      {
        selFileName = selFileName + ".pdf";
      }
      setFilename(selFileName);
    }
  }

  /**
   * Validates the contents of the dialogs input fields. If the selected file
   * exists, it is also checked for validity.
   *
   * @return true, if the input is valid, false otherwise
   */
  public boolean performValidate()
  {
    getStatusBar().clear();

    final String filename = getFilename();
    if (filename.trim().length() == 0)
    {
      getStatusBar().setStatus(JStatusBar.TYPE_ERROR,
              getResources().getString("pdfsavedialog.targetIsEmpty"));
      return false;
    }
    final File f = new File(filename);
    if (f.exists())
    {
      if (f.isFile() == false)
      {
        getStatusBar().setStatus(JStatusBar.TYPE_ERROR,
                getResources().getString("pdfsavedialog.targetIsNoFile"));
        return false;
      }
      if (f.canWrite() == false)
      {
        getStatusBar().setStatus(JStatusBar.TYPE_ERROR,
                getResources().getString("pdfsavedialog.targetIsNotWritable"));
        return false;
      }

      final String message = MessageFormat.format(getResources().getString
              ("pdfsavedialog.targetOverwriteWarning"),
              new Object[]{filename});
      getStatusBar().setStatus(JStatusBar.TYPE_WARNING, message);
    }

    if (getEncryptionValue().equals(PDFOutputTarget.SECURITY_ENCRYPTION_128BIT)
            || getEncryptionValue().equals(
            PDFOutputTarget.SECURITY_ENCRYPTION_40BIT))
    {
      if (txUserPassword.getText().equals(
              txConfUserPassword.getText()) == false)
      {
        getStatusBar().setStatus(JStatusBar.TYPE_ERROR,
                getResources().getString("pdfsavedialog.userpasswordNoMatch"));
        return false;
      }
      if (txOwnerPassword.getText().equals(
              txConfOwnerPassword.getText()) == false)
      {
        getStatusBar().setStatus(JStatusBar.TYPE_ERROR,
                getResources().getString("pdfsavedialog.ownerpasswordNoMatch"));
        return false;
      }
    }

    return true;
  }


  protected boolean performConfirm()
  {
    final String filename = getFilename();
    final File f = new File(filename);
    if (f.exists())
    {
      final String key1 = "pdfsavedialog.targetOverwriteConfirmation";
      final String key2 = "pdfsavedialog.targetOverwriteTitle";
      if (JOptionPane.showConfirmDialog(this,
              MessageFormat.format(getResources().getString(key1),
                      new Object[]{getFilename()}),
              getResources().getString(key2),
              JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE)
              == JOptionPane.NO_OPTION)
      {
        return false;
      }
    }

    if (getEncryptionValue().equals(PDFOutputTarget.SECURITY_ENCRYPTION_128BIT)
            || getEncryptionValue().equals(
            PDFOutputTarget.SECURITY_ENCRYPTION_40BIT))
    {
      if (txOwnerPassword.getText().trim().length() == 0)
      {
        if (JOptionPane.showConfirmDialog(this,
                getResources().getString("pdfsavedialog.ownerpasswordEmpty"),
                getResources().getString("pdfsavedialog.warningTitle"),
                JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE)
                == JOptionPane.NO_OPTION)
        {
          return false;
        }
      }
    }
    return true;
  }

  /**
   * Stores the input from the dialog into the report configuration of the
   * report.
   *
   * @param config the report configuration that should receive the new
   *               settings.
   */
  public void storeToConfiguration(final ModifiableConfiguration config)
  {
    config.setConfigProperty
            (PDFOutputTarget.CONFIGURATION_PREFIX + PDFOutputTarget.AUTHOR,
                    getAuthor());
    config.setConfigProperty
            (PDFOutputTarget.CONFIGURATION_PREFIX + PDFOutputTarget.TITLE,
                    getPDFTitle());
    config.setConfigProperty
            (PDFOutputTarget.CONFIGURATION_PREFIX + PDFOutputTarget.SECURITY_ENCRYPTION,
                    getEncryptionValue());
    config.setConfigProperty
            (PDFOutputTarget.CONFIGURATION_PREFIX + PDFOutputTarget.SECURITY_OWNERPASSWORD,
                    getOwnerPassword());
    config.setConfigProperty
            (PDFOutputTarget.CONFIGURATION_PREFIX + PDFOutputTarget.SECURITY_USERPASSWORD,
                    getUserPassword());
    config.setConfigProperty
            (PDFOutputTarget.CONFIGURATION_PREFIX + PDFOutputTarget.SECURITY_ALLOW_ASSEMBLY,
                    String.valueOf(isAllowAssembly()));
    config.setConfigProperty
            (PDFOutputTarget.CONFIGURATION_PREFIX + PDFOutputTarget.SECURITY_ALLOW_COPY,
                    String.valueOf(isAllowCopy()));
    config.setConfigProperty
            (PDFOutputTarget.CONFIGURATION_PREFIX + PDFOutputTarget.SECURITY_ALLOW_DEGRADED_PRINTING,
                    String.valueOf(isAllowDegradedPrinting()));
    config.setConfigProperty
            (PDFOutputTarget.CONFIGURATION_PREFIX + PDFOutputTarget.SECURITY_ALLOW_FILLIN,
                    String.valueOf(isAllowFillIn()));
    config.setConfigProperty
            (PDFOutputTarget.CONFIGURATION_PREFIX + PDFOutputTarget.SECURITY_ALLOW_MODIFY_ANNOTATIONS,
                    String.valueOf(isAllowModifyAnnotations()));
    config.setConfigProperty
            (PDFOutputTarget.CONFIGURATION_PREFIX + PDFOutputTarget.SECURITY_ALLOW_MODIFY_CONTENTS,
                    String.valueOf(isAllowModifyContents()));
    config.setConfigProperty
            (PDFOutputTarget.CONFIGURATION_PREFIX + PDFOutputTarget.SECURITY_ALLOW_PRINTING,
                    String.valueOf(isAllowPrinting()));
    config.setConfigProperty
            (PDFOutputTarget.CONFIGURATION_PREFIX + PDFOutputTarget.SECURITY_ALLOW_SCREENREADERS,
                    String.valueOf(isAllowScreenreaders()));
    config.setConfigProperty
            (PDFOutputTarget.CONFIGURATION_PREFIX + PDFOutputTarget.ENCODING,
                    getEncoding());
  }

  /**
   * Initialises the PDF save dialog from the settings in the report
   * configuration.
   *
   * @param config the report configuration.
   */
  public void initFromConfiguration(final Configuration config)
  {
    setAllowAssembly(parseBoolean(PDFOutputTarget.SECURITY_ALLOW_ASSEMBLY,
            config,
            isAllowAssembly()));
    setAllowCopy(parseBoolean(PDFOutputTarget.SECURITY_ALLOW_COPY, config,
            isAllowCopy()));
    setAllowFillIn(parseBoolean(PDFOutputTarget.SECURITY_ALLOW_FILLIN, config,
            isAllowFillIn()));
    setAllowModifyAnnotations(parseBoolean(
            PDFOutputTarget.SECURITY_ALLOW_MODIFY_ANNOTATIONS,
            config, isAllowModifyAnnotations()));
    setAllowModifyContents(parseBoolean(
            PDFOutputTarget.SECURITY_ALLOW_MODIFY_CONTENTS, config,
            isAllowModifyContents()));
    setAllowScreenreaders(parseBoolean(
            PDFOutputTarget.SECURITY_ALLOW_SCREENREADERS, config,
            isAllowScreenreaders()));

    final boolean printing = parseBoolean(
            PDFOutputTarget.SECURITY_ALLOW_PRINTING, config,
            isAllowPrinting());
    final boolean degraded = parseBoolean(
            PDFOutputTarget.SECURITY_ALLOW_DEGRADED_PRINTING, config,
            isAllowDegradedPrinting());
    setPrintLevel(printing, degraded);

    setEncryptionValue(config.getConfigProperty(
            PDFOutputTarget.CONFIGURATION_PREFIX
                    + PDFOutputTarget.SECURITY_ENCRYPTION,
            getEncryptionValue()));
    setAuthor(config.getConfigProperty(PDFOutputTarget.CONFIGURATION_PREFIX
            + PDFOutputTarget.AUTHOR, getAuthor()));
    setUserPassword(config.getConfigProperty(
            PDFOutputTarget.CONFIGURATION_PREFIX
                    + PDFOutputTarget.SECURITY_USERPASSWORD,
            getUserPassword()));
    setOwnerPassword(config.getConfigProperty(
            PDFOutputTarget.CONFIGURATION_PREFIX
                    + PDFOutputTarget.SECURITY_OWNERPASSWORD,
            getOwnerPassword()));

    encodingModel.ensureEncodingAvailable(config.getConfigProperty
            (PDFOutputTarget.PDFTARGET_ENCODING,
                    EncodingSupport.getPlatformDefaultEncoding()));
    setEncoding(config.getConfigProperty(PDFOutputTarget.CONFIGURATION_PREFIX
            + PDFOutputTarget.ENCODING, getEncoding()));
    setPDFTitle(config.getConfigProperty(PDFOutputTarget.CONFIGURATION_PREFIX +
            PDFOutputTarget.TITLE, getPDFTitle()));
  }

  /**
   * Parses a boolean property from a report configuration.
   *
   * @param key    the property key.
   * @param config the report configuration.
   * @param orgVal the default value.
   * @return true or false.
   */
  private boolean parseBoolean(final String key, final Configuration config,
                               final boolean orgVal)
  {
    final String val = config.getConfigProperty(
            PDFOutputTarget.CONFIGURATION_PREFIX + key,
            String.valueOf(orgVal));
    return (val.equalsIgnoreCase("true"));
  }

  protected String getConfigurationPrefix()
  {
    return "org.jfree.report.modules.gui.pdf.";
  }

  /**
   * For debugging.
   *
   * @param args ignored.
   */
  public static void main(final String[] args)
  {
    final PDFSaveDialog dialog = new PDFSaveDialog();
    dialog.setModal(true);
    dialog.pack();
    dialog.performQueryForExport(new JFreeReport());
    System.exit(0);
  }


}
