/**
 * ========================================
 * <libname> : a free Java <foobar> library
 * ========================================
 *
 * Project Info:  http://www.jfree.org/liblayout/
 * Project Lead:  Thomas Morgner;
 *
 * (C) Copyright 2005, by Object Refinery Limited and Contributors.
 *
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation;
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * ---------
 * StaticDataFactory.java
 * ---------
 *
 * Original Author:  Thomas Morgner;
 * Contributors: -;
 *
 * $Id: StaticDataFactory.java,v 1.1.2.3 2006/07/07 16:18:20 taqua Exp $
 *
 * Changes
 * -------------------------
 * 27.06.2006 : Initial version
 */
package org.jfree.report.modules.misc.datafactory;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import javax.swing.table.TableModel;

import org.jfree.report.DataFactory;
import org.jfree.report.DataRow;
import org.jfree.report.ReportInitialisationException;
import org.jfree.report.util.CSVTokenizer;
import org.jfree.util.ObjectUtilities;

/**
 * This report data factory uses introspection to search for a report data
 * source. The query has the following format:
 * <p/>
 * &lt;full-qualified-classname&gr;#methodName(Parameters)
 *
 * @author Thomas Morgner
 */
public class StaticDataFactory implements DataFactory
{
  public StaticDataFactory()
  {
  }

  /**
   * Queries a datasource. The string 'query' defines the name of the query. The
   * Parameterset given here may contain more data than actually needed.
   * <p/>
   * The dataset may change between two calls, do not assume anything!
   *
   * @param query
   * @param parameters
   * @return
   */
  public TableModel queryData(final String query, final DataRow parameters)
          throws ReportInitialisationException
  {
    final int methodSeparatorIdx = query.indexOf('#');
    final int parameterStartIdx = query.indexOf('(');
    if (parameterStartIdx >= 0)
    {
      if (methodSeparatorIdx > parameterStartIdx)
      {
        throw new ReportInitialisationException("Malformed query: " + query);
      }
    }

    if ((methodSeparatorIdx + 1) >= query.length())
    {
      throw new ReportInitialisationException("Malformed query: " + query);
    }

    if (methodSeparatorIdx == -1)
    {
      final String[] parameterNames;
      if (parameterStartIdx == -1)
      {
        parameterNames = new String[0];
      }
      else
      {
        parameterNames = createParameterList(query, parameterStartIdx);
      }

      try
      {
        Constructor c = findCallableConstructor(query, parameterNames.length);

        Object[] params = new Object[parameterNames.length];
        for (int i = 0; i < parameterNames.length; i++)
        {
          final String name = parameterNames[i];
          params[i] = parameters.get(name);
        }
        return (TableModel) c.newInstance(params);
      }
      catch (Exception e)
      {
        throw new ReportInitialisationException
                ("Unable to instantiate class for non static call.");
      }
    }

    return createComplexTableModel
            (query, methodSeparatorIdx, parameterStartIdx, parameters);
  }

  private TableModel createComplexTableModel(final String query,
                                             final int methodSeparatorIdx,
                                             final int parameterStartIdx,
                                             final DataRow parameters)
          throws ReportInitialisationException
  {
    final String className = query.substring(0, methodSeparatorIdx);
    final String methodName;
    final String[] parameterNames;
    if (parameterStartIdx == -1)
    {
      // no parameters. Nice.
      parameterNames = new String[0];
      methodName = query.substring(methodSeparatorIdx + 1, query.length());
    }
    else
    {
      methodName = query.substring(methodSeparatorIdx + 1, parameterStartIdx);
      parameterNames = createParameterList(query, parameterStartIdx);
    }

    try
    {
      Method m = findCallableMethod(className, methodName, parameterNames.length);
      Object[] params = new Object[parameterNames.length];
      for (int i = 0; i < parameterNames.length; i++)
      {
        final String name = parameterNames[i];
        params[i] = parameters.get(name);
      }

      if (Modifier.isStatic(m.getModifiers()))
      {
        return (TableModel) m.invoke(null, params);
      }
      else
      {
        ClassLoader classLoader = getClassLoader();
        Class c = classLoader.loadClass(className);
        Object o = c.newInstance();
        if (o == null)
        {
          throw new ReportInitialisationException
                  ("Unable to instantiate class for non static call.");
        }
        return (TableModel) m.invoke(o, params);
      }
    }
    catch (ReportInitialisationException rdfe)
    {
      throw rdfe;
    }
    catch (Exception e)
    {
      throw new ReportInitialisationException
              ("Something went terribly wrong: ", e);
    }
  }

  private String[] createParameterList(final String query,
                                       final int parameterStartIdx)
          throws ReportInitialisationException
  {
    final int parameterEndIdx = query.lastIndexOf(')');
    if (parameterEndIdx < parameterStartIdx)
    {
      throw new ReportInitialisationException("Malformed query: " + query);
    }
    final String parameterText =
            query.substring(parameterStartIdx + 1, parameterEndIdx);
    final CSVTokenizer tokenizer = new CSVTokenizer(parameterText);
    final int size = tokenizer.countTokens();
    final String[] parameterNames = new String[size];
    int i = 0;
    while (tokenizer.hasMoreTokens())
    {
      parameterNames[i] = tokenizer.nextToken();
      i += 1;
    }
    return parameterNames;
  }

  protected ClassLoader getClassLoader()
  {
    return ObjectUtilities.getClassLoader(StaticDataFactory.class);
  }

  private Method findCallableMethod(final String className,
                                    final String methodName,
                                    final int paramCount)
          throws ReportInitialisationException
  {
    ClassLoader classLoader = getClassLoader();

    if (classLoader == null)
    {
      throw new ReportInitialisationException("No classloader!");
    }
    try
    {
      Class c = classLoader.loadClass(className);

      Method[] methods = c.getMethods();
      for (int i = 0; i < methods.length; i++)
      {
        final Method method = methods[i];
        if (Modifier.isPublic(method.getModifiers()) == false)
        {
          continue;
        }
        if (Modifier.isAbstract(method.getModifiers()))
        {
          continue;
        }
        if (method.getName().equals(methodName) == false)
        {
          continue;
        }
        final Class returnType = method.getReturnType();
        if (TableModel.class.isAssignableFrom(returnType) == false)
        {
          continue;
        }
        if (method.getParameterTypes().length != paramCount)
        {
          continue;
        }
        return method;
      }
    }
    catch (ClassNotFoundException e)
    {
      throw new ReportInitialisationException("No such Class", e);
    }
    throw new ReportInitialisationException("No such Method");
  }

  private Constructor findCallableConstructor(final String className,
                                              final int paramCount)
          throws ReportInitialisationException
  {
    ClassLoader classLoader = getClassLoader();

    if (classLoader == null)
    {
      throw new ReportInitialisationException("No classloader!");
    }
    try
    {
      Class c = classLoader.loadClass(className);
      if (TableModel.class.isAssignableFrom(c) == false)
      {
        throw new ReportInitialisationException("Wrong type!");
      }

      Constructor[] methods = c.getConstructors();
      for (int i = 0; i < methods.length; i++)
      {
        final Constructor method = methods[i];
        if (Modifier.isPublic(method.getModifiers()) == false)
        {
          continue;
        }
        if (method.getParameterTypes().length != paramCount)
        {
          continue;
        }
        return method;
      }
    }
    catch (ClassNotFoundException e)
    {
      throw new ReportInitialisationException("No such Class", e);
    }
    throw new ReportInitialisationException("No such Method");
  }
}
